import datetime
import time
import grpc
import sys

from django.core.management.base import BaseCommand
from concurrent import futures
from django.utils import autoreload

from django_grpc.utils import add_servicers, extract_handlers, create_server


class Command(BaseCommand):
    help = 'Run gRPC server'

    def add_arguments(self, parser):
        parser.add_argument('--max_workers', type=int, help="Number of workers")
        parser.add_argument('--port', type=int, default=50051, help="Port number to listen")
        parser.add_argument('--autoreload', action='store_true', default=False)
        parser.add_argument('--list-handlers', action='store_true', default=False, help="Print all registered endpoints")

    def handle(self, *args, **options):
        if options['autoreload'] is True:
            self.stdout.write("ATTENTION! Autoreload is enabled!")
            autoreload.main(self._serve, None, options)
        else:
            self._serve(**options)

    def _serve(self, max_workers, port, *args, **kwargs):
        autoreload.raise_last_exception()
        self.stdout.write("Starting server at %s" % datetime.datetime.now())

        server = create_server(max_workers, port)
        server.start()

        self.stdout.write("Server is listening port %s" % port)

        if kwargs['list_handlers'] is True:
            for handler in extract_handlers(server):
                self.stdout.write(self.style.INFO(handler))

        # since server.start() will not block,
        # a sleep-loop is added to keep alive
        try:
            while True:
                time.sleep(86400)
        except KeyboardInterrupt:
            server.stop(0)
            sys.exit(0)






from concurrent import futures

import grpc
from django.utils.module_loading import import_string


def create_server(max_workers, port):
    # create a gRPC server
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))

    add_servicers(server)
    server.add_insecure_port('[::]:%s' % port)
    return server


def add_servicers(server):
    """
    Add servicers to the server
    """
    from django.conf import settings
    for path in settings.GRPC_SERVICERS:
        callback = import_string(path)
        callback(server)


def extract_handlers(server):
    for it in server._state.generic_handlers[0]._method_handlers.values():
        yield it.unary_unary.__qualname__
